﻿using System;
using System.Windows;
using System.Windows.Input;
using ThinkGeo.MapSuite.SilverlightCore;
using ThinkGeo.MapSuite.SilverlightEdition;

namespace Post7677
{
    public class RestrictExtentInteractiveOverlay : InteractiveOverlay
    {
        private bool isDragging;
        private Point lastMouseMoveScreenPoint;

        public RestrictExtentInteractiveOverlay()
            : this(null)
        { }

        public RestrictExtentInteractiveOverlay(RectangleShape restrictExtent)
        {
            RestrictExtent = restrictExtent;
        }

        public RectangleShape RestrictExtent { get; set; }

        protected override void InitializeCore() { }

        protected override InteractiveResult MouseDownCore(InteractionArguments interactionArguments)
        {
            isDragging = true;
            lastMouseMoveScreenPoint = new Point(interactionArguments.ScreenX, interactionArguments.ScreenY);
            return base.MouseDownCore(interactionArguments);
        }

        protected override InteractiveResult MouseUpCore(InteractionArguments interactionArguments)
        {
            isDragging = false;
            return base.MouseUpCore(interactionArguments);
        }

        protected override InteractiveResult MouseMoveCore(InteractionArguments interactionArguments)
        {
            InteractiveResult result = base.MouseMoveCore(interactionArguments);

            if (isDragging && RestrictExtent != null && ModifierKeys.Shift != Keyboard.Modifiers)
            {
                double currentResolution = GetResolution(interactionArguments.CurrentExtent, interactionArguments.MapWidth, interactionArguments.MapHeight);
                double restrictResolution = GetResolution(RestrictExtent, interactionArguments.MapWidth, interactionArguments.MapHeight);

                if (restrictResolution <= currentResolution)
                {
                    result.NewCurrentExtent = RestrictExtent;
                }
                else
                {
                    double offsetX = lastMouseMoveScreenPoint.X - interactionArguments.ScreenX;
                    double offsetY = interactionArguments.ScreenY - lastMouseMoveScreenPoint.Y;
                    lastMouseMoveScreenPoint = new Point(interactionArguments.ScreenX, interactionArguments.ScreenY);

                    PointShape center = interactionArguments.CurrentExtent.GetCenterPoint();
                    center = new PointShape(center.X + offsetX * currentResolution, center.Y + offsetY * currentResolution);
                    RectangleShape targetExtent = GetBoundingBox(center, currentResolution, interactionArguments.MapWidth, interactionArguments.MapHeight);
                    RectangleShape adjuestExtent = AdjustExtent(targetExtent, interactionArguments.MapWidth, interactionArguments.MapHeight);

                    result.NewCurrentExtent = adjuestExtent;
                    result.ProcessOtherOverlays = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;
                }
            }

            return result;
        }

        private RectangleShape AdjustExtent(RectangleShape targetExtent, double mapWidth, double mapHeight)
        {
            double resolution = GetResolution(targetExtent, mapWidth, mapHeight);
            double gapWidth = resolution * mapWidth * .5;
            double gapHeight = resolution * mapHeight * .5;
            double newLeft = RestrictExtent.LowerLeftPoint.X + gapWidth;
            double newTop = RestrictExtent.UpperLeftPoint.Y - gapHeight;
            double newRight = RestrictExtent.LowerRightPoint.X - gapWidth;
            double newBottom = RestrictExtent.LowerRightPoint.Y + gapHeight;

            if (newLeft < newRight && newTop > newBottom)
            {
                RectangleShape centerRect = new RectangleShape(newLeft, newTop, newRight, newBottom);
                PointShape targetCenter = targetExtent.GetCenterPoint();
                double newCenterX = targetCenter.X;
                double newCenterY = targetCenter.Y;

                if (targetCenter.X < centerRect.LowerLeftPoint.X)
                {
                    newCenterX = centerRect.LowerLeftPoint.X;
                }
                else if (targetCenter.X > centerRect.LowerRightPoint.X)
                {
                    newCenterX = centerRect.LowerRightPoint.X;
                }

                if (targetCenter.Y > centerRect.UpperLeftPoint.Y)
                {
                    newCenterY = centerRect.UpperLeftPoint.Y;
                }
                else if (targetCenter.Y < centerRect.LowerLeftPoint.Y)
                {
                    newCenterY = centerRect.LowerLeftPoint.Y;
                }
                return GetBoundingBox(new PointShape(newCenterX, newCenterY), resolution, mapWidth, mapHeight);
            }
            else return GetBoundingBox(targetExtent.GetCenterPoint(), resolution, mapWidth, mapHeight);
        }

        private static double GetResolution(RectangleShape extent, double screenWidth, double screenHeight)
        {
            return Math.Max(extent.Width / screenWidth, extent.Height / screenHeight);
        }

        private static RectangleShape GetBoundingBox(PointShape center, double resolution, double screenWidth, double screenHeight)
        {
            double halfRslv = resolution * .5;
            double left = center.X - screenWidth * halfRslv;
            double right = center.X + screenWidth * halfRslv;
            double top = center.Y + screenHeight * halfRslv;
            double bottom = center.Y - screenHeight * halfRslv;

            return new RectangleShape(left, top, right, bottom);
        }
    }
}
